#
# Author: Francesco Talpo
#
# This file is part of Shellwrapper.
#
# Shellwrapper is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# Shellwrapper is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Shellwrapper. If not, see <https://www.gnu.org/licenses/>.
#


from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from .logger import LoggerStyle, StyledLogger, CommandLogger, CommandStyle
from .handlers.rich import RichHandler

if TYPE_CHECKING:
    from typing import Iterable

    from ..style import TextStyle


def getStyledLogger(
    name: str | None = None,
    *,
    level: logging._Level = logging.INFO,
    handlers: Iterable[logging.Handler] | None = None,
    debug: TextStyle | None = None,
    info: TextStyle | None = None,
    warning: TextStyle | None = None,
    error: TextStyle | None = None,
    critical: TextStyle | None = None,
) -> StyledLogger:
    style = LoggerStyle(
        debug=debug,
        info=info,
        warn=warning,
        error=error,
        critical=critical,
    )

    hdl = handlers or []
    logger = logging.getLogger(name)
    logger.setLevel(level)
    for handler in hdl:
        logger.addHandler(handler)

    rich = RichHandler(level)
    logger.addHandler(rich)

    return StyledLogger(logger, log_style=style)


def getCommandLogger(
    name: str | None = None,
    *,
    level: logging._Level = logging.INFO,
    handlers: Iterable[logging.Handler] | None = None,
    debug: TextStyle | None = None,
    info: TextStyle | None = None,
    warning: TextStyle | None = None,
    error: TextStyle | None = None,
    critical: TextStyle | None = None,
    command: TextStyle | None = None,
    stdout: TextStyle | None = None,
    stderr: TextStyle | None = None,
) -> CommandLogger:
    style = LoggerStyle(
        debug=debug,
        info=info,
        warn=warning,
        error=error,
        critical=critical,
    )
    cmd_style = CommandStyle(
        command=command,
        stdout=stdout,
        stderr=stderr,
    )

    hdl = handlers or []
    logger = logging.getLogger(name)
    logger.setLevel(level)
    for handler in hdl:
        logger.addHandler(handler)

    rich = RichHandler(level)
    logger.addHandler(rich)

    return CommandLogger(logger, log_style=style, cmd_style=cmd_style)
