#
# Author: Francesco Talpo
#
# This file is part of Shellwrapper.
#
# Shellwrapper is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# Shellwrapper is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Shellwrapper. If not, see <https://www.gnu.org/licenses/>.
#


from __future__ import annotations

import logging
from typing import TYPE_CHECKING

from ..style import TextStyle, STYLE_KEY

if TYPE_CHECKING:
    from typing import Any


class LoggerStyle:

    def __init__(
        self,
        *,
        debug: TextStyle | None = None,
        info: TextStyle | None = None,
        warn: TextStyle | None = None,
        error: TextStyle | None = None,
        critical: TextStyle | None = None,
    ) -> None:
        self.debug = debug or TextStyle(color="light_blue")
        self.info = info or TextStyle()
        self.warn = warn or TextStyle(color="yellow")
        self.error = error or TextStyle(color="red")
        self.critical = critical or TextStyle(color="red", bold=True)


class CommandStyle:

    def __init__(
        self,
        *,
        command: TextStyle | None = None,
        stdout: TextStyle | None = None,
        stderr: TextStyle | None = None,
    ) -> None:
        self.command = command or TextStyle(bold=True)
        self.stdout = stdout or TextStyle(indentation=4)
        self.stderr = stderr or TextStyle(indentation=4, color="red")


class StyledLogger:
    def __init__(
        self,
        logger: logging.Logger = logging.getLogger(),
        *,
        log_style: LoggerStyle | None = None,
    ) -> None:
        self._log_style = log_style or LoggerStyle()
        self._logger = logger

    def __getattr__(self, name: str) -> Any:
        try:
            return getattr(self._logger, name)
        except AttributeError:
            raise AttributeError(f"CommandLogger has no attribute {name}")

    def setLogStyle(self, style: LoggerStyle) -> None:
        self._log_style = style

    def debug(self, *args, **kwargs) -> None:
        return self._logger.debug(
            *args, **kwargs, extra={STYLE_KEY: self._log_style.debug}
        )

    def info(self, *args, **kwargs) -> None:
        return self._logger.info(
            *args, **kwargs, extra={STYLE_KEY: self._log_style.info}
        )

    def warning(self, *args, **kwargs) -> None:
        return self._logger.warning(
            *args, **kwargs, extra={STYLE_KEY: self._log_style.warn}
        )

    def error(self, *args, **kwargs) -> None:
        return self._logger.error(
            *args, **kwargs, extra={STYLE_KEY: self._log_style.error}
        )

    def critical(self, *args, **kwargs) -> None:
        return self._logger.critical(
            *args, **kwargs, extra={STYLE_KEY: self._log_style.critical}
        )


class CommandLogger(StyledLogger):
    def __init__(
        self,
        logger: logging.Logger = logging.getLogger(),
        *,
        log_style: LoggerStyle | None = None,
        cmd_style: CommandStyle | None = None,
    ) -> None:
        super().__init__(logger, log_style=log_style)
        self._cmd_style = cmd_style or CommandStyle()

    def command(self, msg: str, *args, **kwargs) -> None:
        self._logger.info(
            ">>> " + msg,
            *args,
            extra={STYLE_KEY: self._cmd_style.command},
            **kwargs,
        )

    def stdout(self, msg: str, *args, **kwargs) -> None:
        self._logger.info(
            msg,
            *args,
            extra={STYLE_KEY: self._cmd_style.stdout},
            **kwargs,
        )

    def stderr(self, msg: str, *args, **kwargs) -> None:
        self._logger.error(
            msg,
            *args,
            extra={STYLE_KEY: self._cmd_style.stderr},
            **kwargs,
        )
