#
# Author: Francesco Talpo
#
# This file is part of Shellwrapper.
#
# Shellwrapper is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# Shellwrapper is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Shellwrapper. If not, see <https://www.gnu.org/licenses/>.
#


from __future__ import annotations

import asyncio
import os
import logging
from collections import defaultdict
import shlex
import subprocess
from typing import TYPE_CHECKING

from .logging import getCommandLogger

if TYPE_CHECKING:
    from typing import Sequence
    from .logging.logger import CommandLogger


DEFAULT_TIMEOUT = 60


class Shell:

    def __init__(
        self,
        logger: CommandLogger | None = None,
        *,
        quiet: bool = False,
        capture_output: bool = True,
        decode: bool = True,
        check: bool = True,
        timeout: float = DEFAULT_TIMEOUT,
        shell: bool = False,
    ) -> None:
        self._logger = logger or getCommandLogger(level=logging.INFO)
        self.quiet = quiet
        self.capture_output = capture_output
        self.decode = decode
        self.check = check
        self.timeout = timeout
        self.shell = shell

    def _print(
        self,
        command: Sequence[str],
        *,
        env: dict[str, str],
        shell: bool,
        timeout: float,
        check: bool,
        capture: bool,
        decode: bool,
    ) -> subprocess.CompletedProcess:

        async def read_stream(stream, callback):
            while True:
                line = await stream.readline()
                if not line:
                    break
                callback(line)

        async def run_command(
            command: Sequence[str],
        ) -> subprocess.CompletedProcess:
            if shell:
                command = " ".join(command)
                self._logger.command(command)
                process = await asyncio.create_subprocess_shell(
                    command,
                    stdout=asyncio.subprocess.PIPE,
                    stderr=asyncio.subprocess.PIPE,
                    env=env,
                )
            else:
                self._logger.command(" ".join(command))
                process = await asyncio.create_subprocess_exec(
                    command[0],
                    *command[1:],
                    stdout=asyncio.subprocess.PIPE,
                    stderr=asyncio.subprocess.PIPE,
                    env=env,
                )

            captured: dict[str, bytes] = defaultdict(lambda: b"")

            def _handle_stream(name, log_func):

                def _fn(line):
                    log_func(line.decode().strip())
                    captured[name] += line

                return _fn

            stdout_task = asyncio.create_task(
                read_stream(
                    process.stdout,
                    _handle_stream("out", self._logger.stdout),
                )
            )
            stderr_task = asyncio.create_task(
                read_stream(
                    process.stderr,
                    _handle_stream("err", self._logger.stderr),
                )
            )

            _, pending = await asyncio.wait([stdout_task, stderr_task], timeout=timeout)
            retcode = await process.wait()
            if check and retcode:
                raise subprocess.CalledProcessError(retcode, command)

            if pending:
                raise subprocess.TimeoutExpired(command, timeout)

            if not capture:
                return subprocess.CompletedProcess(
                    command, retcode, stdout=None, stderr=None
                )

            if decode and capture:
                return subprocess.CompletedProcess(
                    command,
                    retcode,
                    stdout=captured["out"].decode(),
                    stderr=captured["err"].decode(),
                )

            return subprocess.CompletedProcess(
                command,
                retcode,
                stdout=captured["out"],
                stderr=captured["err"],
            )

        return asyncio.run(run_command(command))

    def _capture_and_print(
        self,
        command: Sequence[str],
        *,
        env: dict[str, str],
        shell: bool,
        timeout: float,
        check: bool,
        decode: bool,
    ) -> subprocess.CompletedProcess:
        return self._print(
            command,
            env=env,
            shell=shell,
            timeout=timeout,
            check=check,
            decode=decode,
            capture=True,
        )

    def _just_print(
        self,
        command: Sequence[str],
        *,
        env: dict[str, str],
        shell: bool,
        timeout: float,
        check: bool,
        decode: bool,
    ) -> subprocess.CompletedProcess:
        return self._print(
            command,
            env=env,
            shell=shell,
            timeout=timeout,
            check=check,
            decode=decode,
            capture=False,
        )

    def _just_capture(
        self,
        command: str | Sequence[str],
        *,
        env: dict[str, str],
        shell: bool,
        timeout: float,
        check: bool,
        decode: bool,
    ) -> subprocess.CompletedProcess:
        return subprocess.run(
            command,
            env=env,
            shell=shell,
            text=decode,
            capture_output=True,
            timeout=timeout,
            check=check,
        )

    @staticmethod
    def _just_run(
        command: str | Sequence[str],
        *,
        env: dict[str, str],
        shell: bool,
        timeout: float,
        check: bool,
        **kwargs,
    ) -> subprocess.CompletedProcess:
        return subprocess.run(
            command,
            env=env,
            shell=shell,
            capture_output=False,
            stdout=subprocess.DEVNULL,
            stderr=subprocess.DEVNULL,
            timeout=timeout,
            check=check,
        )

    def run(
        self,
        command: str | Sequence[str],
        *,
        env: dict[str, str] | None = None,
        shell: bool | None = None,
        timeout: float | None = None,
        check: bool | None = None,
        quiet: bool | None = None,
        capture_output: bool | None = None,
        decode: bool | None = None,
    ):
        env = env if env is not None else os.environ.copy()
        shell = shell if shell is not None else self.shell
        timeout = timeout if timeout is not None else self.timeout
        check = check if check is not None else self.check
        quiet = quiet if quiet is not None else self.quiet
        capture_output = (
            capture_output if capture_output is not None else self.capture_output
        )
        decode = decode if decode is not None else self.decode

        cmd = shlex.split(command) if isinstance(command, str) else command

        match quiet, capture_output:
            case True, True:
                _runfunc = self._just_capture
            case True, False:
                _runfunc = self._just_run
            case False, True:
                _runfunc = self._capture_and_print
            case False, False:
                _runfunc = self._just_print

        return _runfunc(
            cmd, env=env, shell=shell, timeout=timeout, check=check, decode=decode
        )
