#
# Author: Francesco Talpo
#
# This file is part of Shellwrapper.
#
# Shellwrapper is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published
# by the Free Software Foundation, either version 3 of the License,
# or (at your option) any later version.
#
# Shellwrapper is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
# See the GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Shellwrapper. If not, see <https://www.gnu.org/licenses/>.
#


from __future__ import annotations

from typing import TYPE_CHECKING

from rich.console import Console, RenderableType
from rich.padding import Padding
from rich.prompt import Prompt
from rich.style import Style
from rich.text import Text

from .style import TextStyle

if TYPE_CHECKING:
    from typing import Self


def stylize(
    msg: str,
    text_style: TextStyle,
) -> Text:
    text = Text(msg)
    if text_style:
        if text_style.indentation:
            text.pad_left(text_style.indentation)
        style = Style(
            color=text_style.color,
            bgcolor=text_style.background_color,
            bold=text_style.bold,
            underline=text_style.underline,
            italic=text_style.italic,
            strike=text_style.strikethrough,
        )
        text.stylize(style)
    return text


class RichTty:

    def __init__(
        self,
        style: TextStyle | None = None,
        console: Console | None = None,
        *,
        linespaces: int = 1,
    ) -> None:
        self._linespaces = linespaces
        self._console = console or Console()
        self._style = style or TextStyle()
        self._prompt = Prompt()

    def padding(self, msg: RenderableType, style: TextStyle) -> Padding:
        # TODO: fix
        return Padding(msg, (0, style.indentation))

    def print(
        self,
        msg: str,
        *,
        indentation: int = 0,
        color: str | None = None,
        background_color: str | None = None,
        bold: bool = False,
        italic: bool = False,
        underline: bool = False,
        strikethrough: bool = False,
    ) -> None:
        if any(
            (
                indentation,
                color,
                background_color,
                bold,
                italic,
                underline,
                strikethrough,
            )
        ):
            sty = TextStyle(
                0,
                color,
                background_color,
                bold,
                italic,
                underline,
                strikethrough,
            )
        else:
            sty = self._style
        sty_msg = stylize(msg, sty)
        padded = self.padding(sty_msg, sty)
        self._console.print(padded)

    def print_renderable(self, rd: RenderableType, indentation: int = 0) -> None:
        msg = self.padding(rd, TextStyle(indentation=indentation))
        self._console.print(msg)

    def error(self, msg: str) -> None:
        self.print(f"✖  {msg}", color="red1", bold=True)

    def fatal(self, msg: str) -> None:
        self.error(msg)
        exit(1)

    def die(self, msg: str | Exception) -> None:
        if isinstance(msg, Exception):
            from rich.traceback import install

            install()
            self._console.print_exception()
            exit(1)

        self.print(msg)
        exit(1)

    def success(self, msg: str) -> None:
        self.print(f"✔  {msg}", color="green", bold=True)

    def prompt(self, msg: str, choices: list[str] | None = None, **kwargs) -> str:
        if kwargs:
            sty = TextStyle(**kwargs)
        else:
            sty = self._style
        sty_msg = stylize(msg, sty)
        return self._prompt.ask(sty_msg, console=self._console, choices=choices)

    def __enter__(self) -> Self:
        self.print("")
        return self

    def __exit__(self, exc_type, exc_val, exc_tb) -> None:
        self.print("")
        if exc_type is KeyboardInterrupt:
            self.fatal("Aborted by the user; exiting.")
